@@ -1,6 +1,6 @@
 // effects.c
 // LiVES (lives-exe)
-// (c) G. Finch 2003 - 2008
+// (c) G. Finch 2003 - 2010
 // Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
@@ -9,9 +9,17 @@
 #include <unistd.h>
 #include <stdlib.h>
 
+#ifdef HAVE_SYSTEM_WEED
+#include "weed/weed.h"
+#include "weed/weed-palettes.h"
+#include "weed/weed-effects.h"
+#include "weed/weed-host.h"
+#else
 #include "../libweed/weed.h"
+#include "../libweed/weed-palettes.h"
 #include "../libweed/weed-effects.h"
 #include "../libweed/weed-host.h"
+#endif
 
 #include "main.h"
 #include "effects.h"
@@ -54,6 +62,7 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
   gint current_file=mainw->current_file;
 
   gint new_file=current_file;
+  gint ldfile;
 
   gboolean got_no_frames=FALSE;
   gchar *tmp;
@@ -119,7 +128,6 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
     cfile->undo_start=oundo_start;
     cfile->undo_end=oundo_end;
     mainw->current_file=current_file;
-    mainw->is_generating=FALSE;
     return TRUE;
   }
 
@@ -134,6 +142,7 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
     }
     else {
       if (rfx->num_in_channels==0) {
+	mainw->no_switch_dprint=TRUE;
 	if (mainw->gen_to_clipboard) {
 	  tmp=g_strdup(_("%s to clipboard..."));
 	  text=g_strdup_printf(tmp,_(rfx->action_desc));
@@ -149,9 +158,14 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
       }
     }
   }
+
+  if (!mainw->no_switch_dprint) d_print(""); // force switch text
+  ldfile=mainw->last_dprint_file;
+
   d_print(text);
   g_free(text);
   g_free(tmp);
+  mainw->last_dprint_file=ldfile;
 
   cfile->redoable=cfile->undoable=FALSE;
   gtk_widget_set_sensitive (mainw->redo, FALSE);
@@ -196,12 +210,14 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
   else cfile->fx_frame_pump=0;
 
   if (!do_progress_dialog(TRUE,TRUE,effectstring)||mainw->error) {
+    mainw->last_dprint_file=ldfile;
     do_rfx_cleanup(rfx);
     mainw->show_procd=TRUE;
     mainw->keep_pre=FALSE;
     if (mainw->error) {
       do_error_dialog (mainw->msg);
       d_print_failed();
+      mainw->last_dprint_file=ldfile;
     }
     cfile->undo_start=oundo_start;
     cfile->undo_end=oundo_end;
@@ -210,29 +226,39 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
     mainw->resizing=FALSE;
     cfile->nokeep=FALSE;
     cfile->fx_frame_pump=0;
+
     if (rfx->num_in_channels==0) {
       mainw->suppress_dprint=TRUE;
       close_current_file(current_file);
       mainw->suppress_dprint=FALSE;
     }
     else mainw->current_file=current_file;
+
     mainw->is_generating=FALSE;
+    mainw->no_switch_dprint=FALSE;
+
+    if (mainw->multitrack!=NULL) {
+      mainw->pre_src_file=-1;
+    }
+
     return FALSE;
   }
 
   do_rfx_cleanup(rfx);
 
-  mainw->is_generating=FALSE;
   mainw->resizing=FALSE;
   cfile->nokeep=FALSE;
   cfile->fx_frame_pump=0;
-  cfile->changed=TRUE;
-  gtk_widget_set_sensitive (mainw->undo, TRUE);
-  if (rfx->num_in_channels>0) cfile->undoable=TRUE;
-  cfile->pb_fps=old_pb_fps;
-  mainw->internal_messaging=FALSE;
-  if (rfx->num_in_channels>0) gtk_widget_set_sensitive (mainw->select_last, TRUE);
-  if (rfx->num_in_channels>0) set_undoable (_ (rfx->menu_text),TRUE);
+
+  if (!mainw->gen_to_clipboard) {
+    gtk_widget_set_sensitive (mainw->undo, TRUE);
+    if (rfx->num_in_channels>0) cfile->undoable=TRUE;
+    cfile->pb_fps=old_pb_fps;
+    mainw->internal_messaging=FALSE;
+    if (rfx->num_in_channels>0) gtk_widget_set_sensitive (mainw->select_last, TRUE);
+    if (rfx->num_in_channels>0) set_undoable (_ (rfx->menu_text),TRUE);
+  }
+
   mainw->show_procd=TRUE;
   
   if (rfx->props&RFX_PROPS_MAY_RESIZE||rfx->num_in_channels==0) {
@@ -265,11 +291,13 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
       cfile->vsize=weed_get_int_value(first_out,"height",&error);
     }
 
-    if (cfile->hsize==cfile->ohsize&&cfile->vsize==cfile->ovsize) cfile->undo_action=UNDO_EFFECT;
-    else {
-      save_clip_value(mainw->current_file,CLIP_DETAILS_WIDTH,&cfile->hsize);
-      save_clip_value(mainw->current_file,CLIP_DETAILS_HEIGHT,&cfile->vsize);
-      cfile->undo_action=UNDO_RESIZABLE;
+    if (rfx->num_in_channels>0) {
+      if (cfile->hsize==cfile->ohsize&&cfile->vsize==cfile->ovsize) cfile->undo_action=UNDO_EFFECT;
+      else {
+	save_clip_value(mainw->current_file,CLIP_DETAILS_WIDTH,&cfile->hsize);
+	save_clip_value(mainw->current_file,CLIP_DETAILS_HEIGHT,&cfile->vsize);
+	cfile->undo_action=UNDO_RESIZABLE;
+      }
     }
   }
 
@@ -283,86 +311,81 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
     mainw->keep_pre=FALSE;
   }
 
-  if (rfx->num_in_channels!=2) {
-
-    if (rfx->num_in_channels==0) {
-      if (rfx->props&RFX_PROPS_BATCHG) {
-	// batch mode generators need some extra processing
-	gchar *imgdir=g_strdup_printf("%s%s",prefs->tmpdir,cfile->handle);
-	gint img_file=mainw->current_file;
+  if (rfx->num_in_channels==0) {
+    if (rfx->props&RFX_PROPS_BATCHG) {
+      // batch mode generators need some extra processing
+      gchar *imgdir=g_strdup_printf("%s%s",prefs->tmpdir,cfile->handle);
+      gint img_file=mainw->current_file;
+      
+      mainw->suppress_dprint=TRUE;
+      open_file_sel(imgdir,0,0);
+      g_free(imgdir);
+      new_file=mainw->current_file;
+      
+      if (new_file!=img_file) {
+	mainw->current_file=img_file;
 	
-	mainw->suppress_dprint=TRUE;
-	open_file_sel(imgdir,0,0);
-	g_free(imgdir);
-	new_file=mainw->current_file;
+	g_snprintf(mainw->files[new_file]->name,256,"%s",cfile->name);
+	g_snprintf(mainw->files[new_file]->file_name,256,"%s",cfile->file_name);
+	set_menu_text(mainw->files[new_file]->menuentry,cfile->name,FALSE);
 	
-	if (new_file!=img_file) {
-	  mainw->current_file=img_file;
-	
-	  g_snprintf(mainw->files[new_file]->name,256,"%s",cfile->name);
-	  g_snprintf(mainw->files[new_file]->file_name,256,"%s",cfile->file_name);
-	  set_menu_text(mainw->files[new_file]->menuentry,cfile->name,FALSE);
-	  
-	  mainw->files[new_file]->fps=mainw->files[new_file]->pb_fps=cfile->fps;
-	}
-	else got_no_frames=TRUE;
-
-	close_current_file(current_file);
-	mainw->suppress_dprint=FALSE;
-	
-	if (!got_no_frames) mainw->current_file=new_file;
+	mainw->files[new_file]->fps=mainw->files[new_file]->pb_fps=cfile->fps;
       }
-      else {
-	gchar *tfile=g_strdup_printf("%s/%s/%08d.%s",prefs->tmpdir,cfile->handle,cfile->frames,prefs->image_ext);
+      else got_no_frames=TRUE;
+      
+      close_current_file(current_file);
+      mainw->suppress_dprint=FALSE;
+      
+      if (!got_no_frames) mainw->current_file=new_file;
+    }
+    else {
+      gchar *tfile=g_strdup_printf("%s/%s/%08d.%s",prefs->tmpdir,cfile->handle,cfile->frames,prefs->image_ext);
 
-	if (!g_file_test (tfile, G_FILE_TEST_EXISTS)) {
-	  get_frame_count(mainw->current_file);
-	  cfile->end=cfile->frames;
-	}
-	g_free(tfile);
-      }
-
-      if (got_no_frames||cfile->frames==0) {
-	if (!mainw->cancelled) {
-	  do_error_dialog(_("\nNo frames were generated.\n"));
-	  d_print_failed();
-	}
-	else d_print_cancelled();
-	if (!got_no_frames) {
-	  mainw->suppress_dprint=TRUE;
-	  close_current_file(current_file);
-	  mainw->suppress_dprint=FALSE;
-	}
-	return FALSE;
+      if (!g_file_test (tfile, G_FILE_TEST_EXISTS)) {
+	get_frame_count(mainw->current_file);
+	cfile->end=cfile->frames;
       }
+      g_free(tfile);
     }
 
-    if (rfx->num_in_channels!=0||mainw->gen_to_clipboard) {
-      if (rfx->num_in_channels==0) {
-
-	// here we will copy all values to the clipboard, including the handle
-	// then close the current file without deleting the frames
-
-	init_clipboard();
-
-	w_memcpy(clipboard,cfile,sizeof(file));
-	cfile->is_loaded=TRUE;
-	mainw->suppress_dprint=TRUE;
-	mainw->only_close=TRUE;
-
-	close_current_file(current_file);
-
-	mainw->suppress_dprint=FALSE;
-	mainw->only_close=FALSE;
-
-	mainw->current_file=0;
-
-	mainw->untitled_number--;
-
+    if (got_no_frames||cfile->frames==0) {
+      mainw->is_generating=FALSE;
+      if (!mainw->cancelled) {
+	do_error_dialog(_("\nNo frames were generated.\n"));
+	d_print_failed();
       }
-      if (cfile->undo_action==UNDO_RESIZABLE) mainw->current_file=0; // force a resize
-      if (current_file!=-1) switch_to_file (mainw->current_file,current_file);
-      else mainw->current_file=-1;
+      else d_print_cancelled();
+      if (!got_no_frames) {
+	mainw->suppress_dprint=TRUE;
+	close_current_file(current_file);
+	mainw->suppress_dprint=FALSE;
+      }
+      mainw->last_dprint_file=ldfile;
+      mainw->no_switch_dprint=FALSE;
+      if (mainw->multitrack!=NULL) mainw->current_file=mainw->multitrack->render_file;
+      return FALSE;
+    }
+
+    if (mainw->gen_to_clipboard) {
+      // here we will copy all values to the clipboard, including the handle
+      // then close the current file without deleting the frames
+      
+      init_clipboard();
+      
+      w_memcpy(clipboard,cfile,sizeof(file));
+      cfile->is_loaded=TRUE;
+      mainw->suppress_dprint=TRUE;
+      mainw->only_close=TRUE;
+      
+      close_current_file(current_file);
+      
+      mainw->suppress_dprint=FALSE;
+      mainw->only_close=FALSE;
+      
+      new_file=current_file;
+
+      mainw->untitled_number--;
+      
     }
     else {
       if (!(rfx->props&RFX_PROPS_BATCHG)) {
@@ -370,17 +393,36 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
 	cfile->is_loaded=TRUE;
 	add_to_winmenu();
 	save_clip_values(new_file);
-	if (prefs->crash_recovery) add_to_recovery_file(mainw->files[new_file]->handle);
+	if (prefs->crash_recovery) add_to_recovery_file(cfile->handle);
+
+	if (mainw->multitrack!=NULL) {
+	  mt_init_clips(mainw->multitrack,mainw->current_file,TRUE);
+	  mt_clip_select(mainw->multitrack,TRUE);
+	}
+
       }
-      cfile->changed=TRUE;
-      switch_to_file ((mainw->current_file=0),new_file);
+      
 #ifdef ENABLE_OSC
     lives_osc_notify(LIVES_OSC_NOTIFY_CLIP_OPENED,"");
 #endif
     }
+    mainw->is_generating=FALSE;
+  }
+
+  if (!mainw->gen_to_clipboard) cfile->changed=TRUE;
+  if (mainw->multitrack==NULL) {
+    if (new_file!=-1) switch_to_file ((mainw->current_file=0),new_file);
+  }
+  else {
+    mainw->current_file=mainw->multitrack->render_file;
+    mainw->pre_src_file=-1;
   }
 
   d_print_done();
+  mainw->no_switch_dprint=FALSE;
+  mainw->gen_to_clipboard=FALSE;
+  mainw->last_dprint_file=ldfile;
+  
   return TRUE;
 }
 
@@ -445,15 +487,8 @@ gint realfx_progress (gboolean reset) {
 
   g_snprintf(oname,256,"%s/%s/%08d.mgk",prefs->tmpdir,cfile->handle,i);
 
-  if (cfile->img_type==IMG_TYPE_JPEG) {
-    gdk_pixbuf_save (pixbuf, oname, "jpeg", &error,"quality", "100", NULL);
-  }
-  else if (cfile->img_type==IMG_TYPE_PNG) {
-    gdk_pixbuf_save (pixbuf, oname, "png", &error, NULL);
-  }
-  else {
-    //gdk_pixbuf_save_to_callback(...);
-  }
+  lives_pixbuf_save (pixbuf, oname, cfile->img_type, 100, &error);
+
   gdk_pixbuf_unref (pixbuf);
   
 
@@ -468,7 +503,7 @@ gint realfx_progress (gboolean reset) {
     mainw->internal_messaging=FALSE;
 
     if (cfile->clip_type==CLIP_TYPE_FILE) {
-      if (!check_if_non_virtual(cfile)) save_frame_index(mainw->current_file);
+      if (!check_if_non_virtual(mainw->current_file)) save_frame_index(mainw->current_file);
     }
 
   }
@@ -591,7 +626,7 @@ void deinterlace_frame(weed_plant_t *layer, weed_timecode_t tc) {
   
   weed_plant_free(init_event);
   weed_call_deinit_func(deint_instance);
-  weed_free_instance(deint_instance);
+  weed_instance_unref(deint_instance);
 
   g_free(layers);
 }
@@ -608,59 +643,23 @@ void deinterlace_frame(weed_plant_t *layer, weed_timecode_t tc) {
 weed_plant_t *get_blend_layer(weed_timecode_t tc) {
   file *blend_file;
   weed_plant_t *layer;
-  static gdouble blend_add_frames;
-  int oframeno;
-  static weed_timecode_t blend_tc;
-  gint blend_file_step;
+  static weed_timecode_t blend_tc=0;
+  weed_timecode_t ntc=tc;
 
   if (mainw->blend_file==-1||mainw->files[mainw->blend_file]==NULL) return NULL;
   blend_file=mainw->files[mainw->blend_file];
 
   if (mainw->blend_file!=mainw->last_blend_file) {
+    // mainw->last_blend_file is set to -1 on playback start
     mainw->last_blend_file=mainw->blend_file;
-    oframeno=blend_file->frameno;
-    blend_add_frames=0.;
+    blend_tc=tc;
   }
-  else blend_add_frames+=(gdouble)((tc-blend_tc)/U_SEC)*ABS(blend_file->pb_fps);
+  
+  blend_file->last_frameno=blend_file->frameno;
 
-  blend_tc=tc;
+  blend_file->frameno=calc_new_playback_position(mainw->blend_file,blend_tc,&ntc);
 
-  oframeno=blend_file->frameno;
-
-  blend_file_step=blend_file->pb_fps>0?1:-1;
-
-  blend_file->frameno+=(int)(blend_add_frames+.5)*blend_file_step;
-
-  blend_add_frames-=(gdouble)((blend_file->frameno-oframeno)*blend_file_step);
-
-  if (blend_file->frameno>blend_file->end) {
-    blend_add_frames-=blend_file->frameno-blend_file->end;
-    if (blend_add_frames<0.) blend_add_frames=0.;
-    if (mainw->ping_pong) {
-      blend_file->frameno=blend_file->end-(int)blend_add_frames;
-      blend_file->pb_fps=-ABS(blend_file->pb_fps);
-    }
-    else {
-      blend_file->frameno=blend_file->start+(int)blend_add_frames;
-      blend_file->pb_fps=ABS(blend_file->pb_fps);
-    }
-    blend_add_frames-=(int)blend_add_frames;
-    oframeno=blend_file->frameno;
-  }
-  else if (blend_file->frameno<blend_file->start) {
-    blend_add_frames-=blend_file->start-blend_file->frameno;
-    if (blend_add_frames<0.) blend_add_frames=0.;
-    if (mainw->ping_pong) {
-      blend_file->frameno=blend_file->start+(int)blend_add_frames;
-      blend_file->pb_fps=ABS(blend_file->pb_fps);
-    }
-    else {
-      blend_file->frameno=blend_file->end-(int)blend_add_frames;
-      blend_file->pb_fps=-ABS(blend_file->pb_fps);
-    }
-    blend_add_frames-=(int)blend_add_frames;
-    oframeno=blend_file->frameno;
-  }
+  blend_tc=ntc;
 
   layer=weed_plant_new(WEED_PLANT_CHANNEL);
   weed_set_int_value(layer,"clip",mainw->blend_file);
@@ -717,7 +716,7 @@ gboolean rte_on_off_callback (GtkAccelGroup *group, GObject *obj, guint keyval,
     }
   }
   if (mainw->current_file>0&&cfile->play_paused&&!mainw->noswitch) {
-    load_frame_image (cfile->frameno,cfile->last_frameno);
+    load_frame_image (cfile->frameno);
   }
   return TRUE;
 }
@@ -784,7 +783,7 @@ gboolean rtemode_callback_hook (GtkToggleButton *button, gpointer user_data) {
 gboolean swap_fg_bg_callback (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer user_data) {
   gint old_file=mainw->current_file;
 
-  if (mainw->playing_file<0||mainw->num_tr_applied==0||mainw->noswitch||mainw->blend_file==-1||mainw->blend_file==mainw->current_file||mainw->files[mainw->blend_file]==NULL||mainw->preview||mainw->noswitch) {
+  if (mainw->playing_file<1||mainw->num_tr_applied==0||mainw->noswitch||mainw->blend_file==-1||mainw->blend_file==mainw->current_file||mainw->files[mainw->blend_file]==NULL||mainw->preview||mainw->noswitch||(mainw->is_processing&&cfile->is_loaded)) {
     return TRUE;
   }
 
