@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     agint64 with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 */
 
 
@@ -63,6 +63,8 @@ POSSIBILITY OF SUCH DAMAGES.
 #include <signal.h>
 #include <assert.h>
 
+#define QUOTEME(x) #x
+
 /// don't change this unless the backend is changed as well
 /// i.e. $GUI_BOOTSTRAP_FILE in smogrify
 #define BOOTSTRAP_NAME "/tmp/.smogrify"
@@ -555,6 +557,10 @@ typedef struct {
 
   gboolean has_stderr;
 
+  gint nmonitors;
+
+  gint ncpus;
+
 } capability;
 
 
@@ -628,12 +634,6 @@ typedef enum {
 
 // some useful functions
 
-//gui.c
-void  create_LiVES (void);
-void enable_record (void);
-void toggle_record (void);
-void disable_record (void);
-
 // interface.c
 process* create_processing (const gchar *text);
 void add_to_winmenu(void);
@@ -672,8 +672,8 @@ gboolean do_comments_dialog (file *sfile, gchar *filename);
 void do_auto_dialog(const gchar *text, gint type);
 void do_encoder_acodec_error (void);
 void do_encoder_sox_error(void);
-gboolean rdet_suggest_values (gint width, gint height, gdouble fps, gint fps_num, gint fps_denom, gint arate, gint asigned, gboolean anr, gboolean ignore_fps);
-gboolean do_encoder_restrict_dialog (gint width, gint height, gdouble fps, gint fps_num, gint fps_denom, gint arate, gint asigned, gboolean anr);
+gboolean rdet_suggest_values (gint width, gint height, gdouble fps, gint fps_num, gint fps_denom, gint arate, gint asigned, gboolean swap_endian, gboolean anr, gboolean ignore_fps);
+gboolean do_encoder_restrict_dialog (gint width, gint height, gdouble fps, gint fps_num, gint fps_denom, gint arate, gint asigned, gboolean swap_endian, gboolean anr);
 void do_keys_window (void);
 void do_mt_keys_window (void);
 void do_messages_window (void);
@@ -727,8 +727,9 @@ void do_locked_in_vdevs_error(void);
 void do_do_not_close_d (void);
 
 gboolean process_one (gboolean visible);
-void do_threaded_dialog(const gchar *text, gboolean has_cancel);
+void do_threaded_dialog(gchar *translated_text, gboolean has_cancel);
 void end_threaded_dialog(void);
+void threaded_dialog_spin (void);
 void response_ok (GtkButton *button, gpointer user_data);
 void response_cancel (GtkButton *button, gpointer user_data);
 void pump_io_chan(GIOChannel *iochan);
@@ -802,6 +803,7 @@ void load_preview_image(gboolean update_always);
 gboolean pull_frame(weed_plant_t *layer, const gchar *image_ext, weed_timecode_t tc);
 gboolean pull_frame_at_size (weed_plant_t *layer, const gchar *image_ext, weed_timecode_t tc, int width, int height, int target_palette);
 GdkPixbuf *pull_gdk_pixbuf_at_size(gint clip, gint frame, const gchar *image_ext, weed_timecode_t tc, gint width, gint height, GdkInterpType interp);
+GError * lives_pixbuf_save(GdkPixbuf *pixbuf, gchar *fname, lives_image_type_t imgtype, int quality, GError **gerrorptr);
 
 void load_frame_image(gint frame);
 void sensitize(void);
@@ -819,10 +821,13 @@ void set_palette_colours (void);
 void set_main_title(const gchar *filename, gint or_untitled_number);
 void set_record (void);
 
-// main.c - in progress
-weed_plant_t *weed_layer_new_from_video(gint fnum, gint frame);
-
 //gui.c
+//gui.c
+void  create_LiVES (void);
+void enable_record (void);
+void toggle_record (void);
+void disable_record (void);
+void make_custom_submenus(void);
 void fade_background(void);
 void unfade_background(void);
 void block_expose (void);
@@ -831,6 +836,7 @@ void frame_size_update(void);
 void splash_init(void);
 void splash_end(void);
 void splash_msg(const gchar *msg, gdouble pct);
+void add_message_scroller(GtkWidget *conter);
 
 // utils.c
 #ifdef IS_IRIX
@@ -857,11 +863,14 @@ void get_pref_default(const gchar *key, gchar *val, gint maxlen);
 gboolean get_boolean_pref(const gchar *key);
 gdouble get_double_pref(const gchar *key);
 gint get_int_pref(const gchar *key);
+GList *get_list_pref(const gchar *key);
 void get_location(const gchar *exe, gchar *val, gint maxlen);
 void set_pref (const gchar *key, const gchar *value);
+void delete_pref (const gchar *key);
 void set_boolean_pref(const gchar *key, gboolean value);
 void set_double_pref(const gchar *key, gdouble value);
 void set_int_pref(const gchar *key, gboolean value);
+void set_list_pref(const char *key, GList *values);
 gboolean apply_prefs(gboolean skip_warnings);
 void save_future_prefs(void);
 void set_menu_text(GtkWidget *menu, const gchar *text, gboolean use_mnemonic);
@@ -879,10 +888,10 @@ void set_alwaysontop(GtkWidget *window, gboolean ontop); ///< TODO - use for pla
 void colour_equal(GdkColor *c1, const GdkColor *c2);
 void switch_to_int_player(void);
 void switch_to_mplayer(void);
-void switch_aud_to_sox(void);
+void switch_aud_to_sox(gboolean set_pref);
 gboolean switch_aud_to_jack(void);
 gboolean switch_aud_to_pulse(void);
-void switch_aud_to_mplayer(void);
+void switch_aud_to_mplayer(gboolean set_pref);
 void prepare_to_play_foreign(void);
 gboolean after_foreign_play(void);
 gboolean check_file(const gchar *file_name, gboolean check_exists);  ///< check if file exists
@@ -944,6 +953,9 @@ gint calc_new_playback_position(gint fileno, gint64 otc, gint64 *ntc);
 void minimise_aspect_delta (gdouble allowed_aspect,gint hblock,gint vblock,gint hsize,gint vsize,gint *width,gint *height);
 LIVES_INLINE gint get_interp_value(gshort quality);
 LIVES_INLINE GList *g_list_move_to_first(GList *list, GList *item) WARN_UNUSED;
+GList *g_list_delete_string(GList *, char *string) WARN_UNUSED;
+GList *g_list_copy_strings(GList *list);
+gboolean string_lists_differ(GList *, GList *);
 
 
 GList *get_set_list(const gchar *dir);
