@@ -816,7 +816,7 @@ typedef struct {
   GMemVTable alt_vtable;
   void (*free_fn)(gpointer);
 
-  pthread_mutex_t gtk_mutex;  ///< gtk drawing mutex - used by the threaded dialog
+  pthread_mutex_t gtk_mutex;  ///< gtk drawing mutex - no longer used
   pthread_mutex_t interp_mutex;  ///< interpolation mutex - parameter interpolation must be single threaded
 
   pthread_mutex_t abuf_mutex;  ///< used to synch audio buffer request count - shared between audio and video threads
@@ -876,7 +876,6 @@ typedef struct {
   _vid_playback_plugin *vpp;
 
   /// multi-head support
-  gint nmonitors;
   lives_mgeometry_t *mgeom;
 
 
@@ -909,6 +908,10 @@ typedef struct {
   pthread_t sig_pid;
   gchar *sig_file;
 
+  gboolean has_custom_tools;
+  gboolean has_custom_gens;
+  gboolean has_custom_utilities;
+
   /// decoders
   gboolean decoders_loaded;
   GList *decoder_list;
