@@ -1,6 +1,6 @@
 // preferences.h
 // LiVES (lives-exe)
-// (c) G. Finch 2004 - 2009
+// (c) G. Finch 2004 - 2011
 // released under the GNU GPL 3 or later
 // see file ../COPYING or www.gnu.org for licensing details
 
@@ -13,14 +13,14 @@ typedef struct {
   gchar bgcolour[256];
   gboolean stop_screensaver;
   gboolean open_maximised;
-  gchar theme[64];  // the theme name
+  gchar theme[64];  ///< the theme name
 
   gshort pb_quality;
 #define PB_QUALITY_LOW 1
-#define PB_QUALITY_MED 2  // default
+#define PB_QUALITY_MED 2  ///< default
 #define PB_QUALITY_HIGH 3
 
-  _encoder encoder; // from main.h
+  _encoder encoder; ///< from main.h
 
   gshort audio_player;
 #define AUD_PLAYER_SOX 1
@@ -28,12 +28,14 @@ typedef struct {
 #define AUD_PLAYER_JACK 3
 #define AUD_PLAYER_PULSE 4
 
-  // frame quantisation type
+  gchar aplayer[512]; // name, eg. "jack","pulse","sox","mplayer"
+
+  /// frame quantisation type
   gshort q_type;
 #define Q_FILL 1
 #define Q_SMOOTH 1
 
-  gchar tmpdir[256];  // kept in locale encoding
+  gchar tmpdir[256];  ///< kept in locale encoding
 
   // TODO - also use cur_vid_load_dir, etc.
   // utf8 encoding
@@ -54,7 +56,10 @@ typedef struct {
   // if these bits are set, we do not show the warning
 #define WARN_MASK_FPS (1<<0)
 #define WARN_MASK_FSIZE (1<<1)
+
+  /// no longer used
 #define WARN_MASK_SAVE_QUALITY (1<<2)
+
 #define WARN_MASK_SAVE_SET (1<<3)
 #define WARN_MASK_NO_MPLAYER (1<<4)
 #define WARN_MASK_RENDERED_FX (1<<5)
@@ -69,7 +74,7 @@ typedef struct {
 
 #define WARN_MASK_DUPLICATE_SET (1<<12)
 
-#define WARN_MASK_DISCARD_LAYOUT (1<<13)
+#define WARN_MASK_EXIT_MT (1<<13)
 #define WARN_MASK_DISCARD_SET (1<<14)
 #define WARN_MASK_AFTER_DVGRAB (1<<15)
 
@@ -87,10 +92,16 @@ typedef struct {
 
 #define WARN_MASK_MT_BACKUP_SPACE (1<<22)
 
+#define WARN_MASK_LAYOUT_POPUP (1<<23)
+
+#define WARN_MASK_CLEAN_AFTER_CRASH (1<<24)
+
+#define WARN_MASK_NO_PULSE_CONNECT (1<<25)
+
   gchar effect_command[256];
   gchar video_open_command[256];
   gchar audio_play_command[256];
-  gchar cdplay_device[256];  // locale encoding
+  gchar cdplay_device[256];  ///< locale encoding
   gdouble default_fps;
   gint bar_height;
   gboolean pause_effect_during_preview;
@@ -109,6 +120,7 @@ typedef struct {
   gshort sepwin_type;
   gboolean show_player_stats;
   gboolean show_framecount;
+  gboolean show_subtitles;
   gboolean loop_recording;
   gboolean discard_tv;
   gboolean save_directories;
@@ -124,13 +136,13 @@ typedef struct {
   gboolean osc_udp_started;
   guint osc_udp_port;
 
-  gboolean omc_noisy;
+  gboolean omc_noisy; ///< send success/fail
+  gboolean omc_events; ///< send other events
 
-  gshort startup_phase; // 1 = new startup, 2, pre-audio start, 100 = all tests passed
-  const gchar *wm; //window manager name
-  gint ocp; // open_compression_percent : get/set in prefs
+  gshort startup_phase; ///< -1 = fresh install, 1 = tmpdir set, 2, pre-audio start, 3, pre-tests, 100 = all tests passed
+  const gchar *wm; ///<window manager name
+  gint ocp; ///< open_compression_percent : get/set in prefs
 
-  // whether the 'blend_file' (second file in a transition) is fg or bg
   gboolean antialias;
 
   gboolean ignore_tiny_fps_diffs;
@@ -138,11 +150,14 @@ typedef struct {
   gshort rte_keys_virtual;
 
   guint jack_opts;
-#define JACK_OPTS_TRANSPORT_CLIENT (1<<0)
-#define JACK_OPTS_TRANSPORT_MASTER (1<<1)
-#define JACK_OPTS_START_TSERVER (1<<2)
-#define JACK_OPTS_NOPLAY_WHEN_PAUSED (1<<3)
-#define JACK_OPTS_START_ASERVER (1<<4)
+#define JACK_OPTS_TRANSPORT_CLIENT (1<<0)   ///< jack can start/stop
+#define JACK_OPTS_TRANSPORT_MASTER (1<<1)  ///< transport master
+#define JACK_OPTS_START_TSERVER (1<<2)     ///< start transport server
+#define JACK_OPTS_NOPLAY_WHEN_PAUSED (1<<3) ///< play audio even when transport paused
+#define JACK_OPTS_START_ASERVER (1<<4)     ///< start audio server
+#define JACK_OPTS_TIMEBASE_START (1<<5)    ///< jack sets play start position
+#define JACK_OPTS_TIMEBASE_CLIENT (1<<6)    ///< full timebase client
+#define JACK_OPTS_TIMEBASE_MASTER (1<<7)   ///< timebase master (not implemented yet)
 
   gchar jack_tserver[256];
   gchar jack_aserver[256];
@@ -159,9 +174,9 @@ typedef struct {
 #define AUDIO_OPTS_FOLLOW_FPS (1<<1)
 
   gboolean event_window_show_frame_events;
-  gboolean crash_recovery; // TRUE==maintain mainw->recovery file
+  gboolean crash_recovery; ///< TRUE==maintain mainw->recovery file
 
-  gboolean show_rdet; // show render details (frame size, encoder type) before saving to file
+  gboolean show_rdet; ///< show render details (frame size, encoder type) before saving to file
 
   gboolean move_effects;
 
@@ -188,17 +203,22 @@ typedef struct {
   gboolean ar_clipset;
   gboolean ar_layout;
 
-  gchar ar_clipset_name[128]; // utf8 (not converted)
-  gchar ar_layout_name[128];  // utf8 (not converted)
+  gchar ar_clipset_name[128]; ///< utf8 (not converted)
+  gchar ar_layout_name[128];  ///< utf8 (not converted)
 
   gboolean rec_desktop_audio;
 
   gboolean show_gui;
+  gboolean show_splash;
+  gboolean show_playwin;
+
   gboolean osc_start;
 
+  gboolean show_threaded_dialog;
+
   gboolean collate_images;
 
-  gint virt_height; // n screens vert.
+  gint virt_height; ///< n screens vert.
 
   gboolean concat_images;
 
@@ -206,6 +226,7 @@ typedef struct {
 
   gboolean instant_open;
   gboolean auto_deint;
+  gboolean auto_nobord;
 
   gint gui_monitor;
   gint play_monitor;
@@ -220,8 +241,8 @@ typedef struct {
 #define OMC_DEV_FORCE_RAW_MIDI 1<<2
   guint omc_dev_opts;
 
-  gchar omc_js_fname[256];  // utf8
-  gchar omc_midi_fname[256]; // utf8
+  gchar omc_js_fname[256];  ///< utf8
+  gchar omc_midi_fname[256]; ///< utf8
   
   gboolean mouse_scroll_clips;
 
@@ -233,17 +254,90 @@ typedef struct {
   gboolean use_alsa_midi;
 #endif
 
+  gint startup_interface;
+
+#define STARTUP_CE 0
+#define STARTUP_MT 1
+
+  gboolean ce_maxspect;
+
+  gboolean lamp_buttons;
+
+  gboolean autoload_subs;
+
+  glong rec_stop_gb;
+
+  guint max_modes_per_key; ///< maximum effect modes per key
+
+  // autotransitioning in mt
+  gint atrans_fx;
+  gchar def_autotrans[256];
+
+  gint nfx_threads;
+
+  gboolean stream_audio_out;
+  gboolean unstable_fx;
+
+  GList *disabled_decoders;
 } _prefs;
 
+enum {
+    LIST_ENTRY_GUI,
+    LIST_ENTRY_DECODING,
+    LIST_ENTRY_PLAYBACK,
+    LIST_ENTRY_RECORDING,
+    LIST_ENTRY_ENCODING,
+    LIST_ENTRY_EFFECTS,
+    LIST_ENTRY_DIRECTORIES,
+    LIST_ENTRY_WARNINGS,
+    LIST_ENTRY_MISC,
+    LIST_ENTRY_THEMES,
+    LIST_ENTRY_NET,
+    LIST_ENTRY_JACK,
+    LIST_ENTRY_MIDI,
+    LIST_ENTRY_MULTITRACK
+};
+
+enum {
+    LIST_ICON = 0,
+    LIST_ITEM,
+    LIST_NUM,
+    N_COLUMNS
+};
 
 
-// prefs window
+#define PREF_WIN_WIDTH 960
+#define PREF_WIN_HEIGHT 640
+
+
+/// prefs window
 typedef struct {
   gulong encoder_ofmt_fn;
   gulong encoder_name_fn;
   GtkWidget *prefs_dialog;
-  GtkWidget *prefs_notebook;
+  
+  GtkWidget *prefs_list;
+  GtkWidget *prefs_table;
+  GtkWidget *vbox_right_gui;
+  GtkWidget *vbox_right_multitrack;
+  GtkWidget *vbox_right_decoding;
+  GtkWidget *vbox_right_playback;
+  GtkWidget *vbox_right_recording;
+  GtkWidget *vbox_right_encoding;
+  GtkWidget *vbox_right_effects;
+  GtkWidget *table_right_directories;
+  GtkWidget *vbox_right_warnings;
+  GtkWidget *vbox_right_warnings_outer;
+  GtkWidget *vbox_right_misc;
+  GtkWidget *vbox_right_themes;
+  GtkWidget *vbox_right_net;
+  GtkWidget *vbox_right_jack;
+  GtkWidget *vbox_right_midi;
+  GtkWidget *scrollw;
+  GtkWidget *right_shown;
   GtkWidget *cancelbutton;
+  GtkWidget *applybutton;
+  GtkWidget *closebutton;
   GtkWidget *stop_screensaver_check;
   GtkWidget *open_maximised_check;
   GtkWidget *show_tool;
@@ -260,6 +354,7 @@ typedef struct {
   GtkWidget *tmpdir_entry;
   GtkWidget *cdplay_entry;
   GtkWidget *spinbutton_def_fps;
+  GtkWidget *pbq_combo;
   GtkWidget *ofmt_combo;
   GtkWidget *audp_combo;
   GtkWidget *rframes;
@@ -270,6 +365,7 @@ typedef struct {
   GtkWidget *rdesk_audio;
   GtkWidget *encoder_combo;
   GtkWidget *checkbutton_antialias;
+  GtkWidget *checkbutton_threads;
   GtkWidget *checkbutton_warn_fps;
   GtkWidget *checkbutton_warn_mplayer;
   GtkWidget *checkbutton_warn_save_set;
@@ -284,9 +380,10 @@ typedef struct {
   GtkWidget *checkbutton_warn_layout_adel;
   GtkWidget *checkbutton_warn_layout_aalt;
   GtkWidget *checkbutton_warn_layout_ashift;
+  GtkWidget *checkbutton_warn_layout_popup;
   GtkWidget *checkbutton_warn_discard_layout;
-  GtkWidget *checkbutton_warn_save_quality;
   GtkWidget *checkbutton_warn_after_dvgrab;
+  GtkWidget *checkbutton_warn_no_pulse;
   GtkWidget *checkbutton_show_stats;
   GtkWidget *check_xmms_pause;
   GtkWidget *checkbutton_warn_fsize;
@@ -294,26 +391,27 @@ typedef struct {
   GtkWidget *checkbutton_warn_mt_no_jack;
   GtkWidget *checkbutton_warn_yuv4m_open;
   GtkWidget *checkbutton_warn_mt_backup_space;
+  GtkWidget *checkbutton_warn_after_crash;
   GtkWidget *spinbutton_warn_fsize;
   GtkWidget *spinbutton_bwidth;
   GtkWidget *theme_combo;
   GtkWidget *check_midi;
-  GtkWidget *rb_aq_high;
-  GtkWidget *rb_aq_low;
   GtkWidget *ins_speed;
   GtkWidget *jpeg;
   GtkWidget *mt_enter_prompt;
   GtkWidget *spinbutton_ocp;
   GtkWidget *acodec_combo;
-  GtkWidget *acodec_entry;
   GtkWidget *spinbutton_osc_udp;
   GtkWidget *spinbutton_rte_keys;
+  GtkWidget *spinbutton_nfx_threads;
   GtkWidget *enable_OSC;
   GtkWidget *enable_OSC_start;
   GtkWidget *jack_tserver_entry;
   GtkWidget *jack_aserver_entry;
   GtkWidget *checkbutton_jack_master;
   GtkWidget *checkbutton_jack_client;
+  GtkWidget *checkbutton_jack_tb_start;
+  GtkWidget *checkbutton_jack_tb_client;
   GtkWidget *checkbutton_jack_pwp;
   GtkWidget *checkbutton_start_tjack;
   GtkWidget *checkbutton_start_ajack;
@@ -324,6 +422,7 @@ typedef struct {
   GtkWidget *spinbutton_mt_def_fps;
   GtkWidget *spinbutton_mt_undo_buf;
   GtkWidget *spinbutton_mt_ab_time;
+  GtkWidget *spinbutton_rec_gb;
   GtkWidget *mt_autoback_every;
   GtkWidget *mt_autoback_always;
   GtkWidget *mt_autoback_never;
@@ -335,11 +434,12 @@ typedef struct {
   GtkWidget *checkbutton_render_prompt;
   GtkWidget *checkbutton_instant_open;
   GtkWidget *checkbutton_auto_deint;
+  GtkWidget *checkbutton_nobord;
   GtkWidget *checkbutton_concat_images;
   GtkWidget *forcesmon;
-  GtkWidget *pbq_entry;
   GList *pbq_list;
   gchar *audp_name;
+  gchar *orig_audp_name;
   gulong audp_entry_func;
   GtkWidget *checkbutton_omc_js;
   GtkWidget *checkbutton_omc_midi;
@@ -349,22 +449,28 @@ typedef struct {
   GtkWidget *spinbutton_midirpt;
   GtkWidget *alsa_midi;
   GtkWidget *button_midid;
+  GtkWidget *rb_startup_ce;
+  GtkWidget *rb_startup_mt;
+  GtkWidget *jack_int_label;
+  GtkWidget *checkbutton_ce_maxspect;
+  GtkWidget *temp_label;
+  GtkWidget *checkbutton_stream_audio;
+  gboolean needs_restart;
 } _prefsw;
 
-
-
-typedef struct {  // startup overrides from commandline
+/// startup overrides from commandline
+typedef struct {
   gboolean ign_clipset;
   gboolean ign_osc;
-  gboolean ign_jackopts;
   gboolean ign_aplayer;
+  gboolean ign_stmode;
+  gboolean ign_vppdefs;
 } _ign_opts;
 
-
 typedef struct {
   gchar tmpdir[256];
   gchar theme[64];
-  gchar vpp_name[64]; // new video playback plugin
+  gchar vpp_name[64]; ///< new video playback plugin
   gint vpp_fixed_fps_numer;
   gint vpp_fixed_fps_denom;
   gdouble vpp_fixed_fpsd;
@@ -379,16 +485,22 @@ typedef struct {
   gboolean show_recent;
   gboolean show_tool;
   gboolean osc_start;
+  gint startup_interface;
   guint jack_opts;
+
+  gint nfx_threads;
+
+  GList *disabled_decoders;
+  GList *disabled_decoders_new;
+
 } _future_prefs;
 
 _prefs *prefs;
 _future_prefs *future_prefs;
 _prefsw *prefsw;
 
-
 void set_acodec_list_from_allowed (_prefsw *, render_details *);
-void  rdet_acodec_changed (GtkEntry *acodec_entry, gpointer user_data);
+void  rdet_acodec_changed (GtkComboBox *acodec_combo, gpointer user_data);
 
 _prefsw* create_prefs_dialog (void);
 
@@ -396,10 +508,22 @@ gboolean on_prefs_delete_event (GtkWidget *, GdkEvent *, gpointer prefsw);
 
 void on_preferences_activate (GtkMenuItem *, gpointer);
 
-void on_prefs_ok_clicked (GtkButton *, gpointer);
+void on_prefs_close_clicked (GtkButton *, gpointer);
 
-void on_prefs_cancel_clicked (GtkButton *, gpointer);
+void on_prefs_revert_clicked (GtkButton *, gpointer);
 
 void set_vpp(gboolean set_in_prefs);
 
+void on_prefDomainChanged(GtkTreeSelection *, gpointer);
+
+void select_pref_list_row(guint);
+
+void populate_combo_box(GtkComboBox *combo, GList *data);
+
+void set_combo_box_active_string(GtkComboBox *, gchar *active_str);
+
+void prefsw_set_astream_settings(_vid_playback_plugin *);
+
+void apply_button_set_enabled(GtkWidget *widget, gpointer func_data);
+
 #endif
