@@ -1,6 +1,6 @@
 // rte_window.c
 // LiVES (lives-exe)
-// (c) G. Finch 2005
+// (c) G. Finch 2005 - 2010
 // released under the GNU GPL 3 or later
 // see file ../COPYING or www.gnu.org for licensing details
 
@@ -8,15 +8,21 @@
 #include <errno.h>
 #include <sys/stat.h>
 
+#ifdef HAVE_SYSTEM_WEED
+#include "weed/weed.h"
+#include "weed/weed-host.h"
+#include "weed/weed-effects.h"
+#else
 #include "../libweed/weed.h"
 #include "../libweed/weed-host.h"
+#include "../libweed/weed-effects.h"
+#endif
 
 #include "main.h"
 
 #include "support.h"
 #include "rte_window.h"
 #include "effects.h"
-#include "effects-weed.h"
 #include "paramwindow.h"
 
 #define RTE_INFO_WIDTH 350
@@ -71,21 +77,25 @@ void type_label_set_text (gint key, gint mode) {
     gtk_widget_set_sensitive(type_labels[idx],FALSE);
   }
   g_free(type);
-  gtk_widget_set_sensitive(param_buttons[idx],(rte_keymode_get_instance(key+1,mode)!=NULL));
 }
 
 
-void 
-on_rtei_ok_clicked (GtkButton *button, gpointer user_data) {
+void on_rtei_ok_clicked (GtkButton *button, gpointer user_data) {
   gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
 }
 
+
 gboolean on_clear_all_clicked (GtkButton *button, gpointer user_data) {
   int modes=rte_getmodespk();
   int i,j;
 
+  mainw->error=FALSE;
+
   // prompt for "are you sure ?"
-  if (user_data!=NULL) if (!do_warning_dialog_with_check_transient((_("\n\nUnbind all effects from all keys/modes.\n\nAre you sure ?\n\n")),0,GTK_WINDOW(rte_window))) return FALSE;
+  if (user_data!=NULL) if (!do_warning_dialog_with_check_transient((_("\n\nUnbind all effects from all keys/modes.\n\nAre you sure ?\n\n")),0,GTK_WINDOW(rte_window))) {
+      mainw->error=TRUE;
+      return FALSE;
+    }
 
   for (i=0;i<prefs->rte_keys_virtual;i++) {
     if (rte_window!=NULL) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(key_checks[i]),FALSE);
@@ -104,8 +114,43 @@ gboolean on_clear_all_clicked (GtkButton *button, gpointer user_data) {
 }
 
 
-gboolean 
-on_save_keymap_clicked (GtkButton *button, gpointer user_data) {
+static void save_keymap2_file(gchar *kfname) {
+  int i,j;
+  int slen;
+  int version=1;
+  int modes=rte_getmodespk();
+  int kfd=creat(kfname,S_IRUSR|S_IWUSR);
+  gchar *msg;
+  gchar *hashname;
+
+  if (kfd==-1) {
+    msg=g_strdup_printf (_("\n\nUnable to write keymap file\n%s\nError code %d\n"),kfname,errno);
+    do_error_dialog_with_check_transient (msg,FALSE,0,GTK_WINDOW(rte_window));
+    g_free (msg);
+    d_print_failed();
+    return;
+  }
+
+  dummyvar=write(kfd,&version,sizint);
+
+  for (i=1;i<=prefs->rte_keys_virtual;i++) {
+    for (j=0;j<modes;j++) {
+      if (rte_keymode_valid(i,j,TRUE)) {
+	dummyvar=write(kfd,&i,sizint);
+	hashname=g_strdup_printf("Weed%s",make_weed_hashname(rte_keymode_get_filter_idx(i,j),TRUE));
+	slen=strlen(hashname);
+	dummyvar=write(kfd,&slen,sizint);
+	dummyvar=write(kfd,hashname,slen);
+	g_free(hashname);
+	write_key_defaults(kfd,i-1,j);
+      }
+    }
+  }
+  close(kfd);
+}
+
+
+gboolean on_save_keymap_clicked (GtkButton *button, gpointer user_data) {
   // quick and dirty implementation
   int modes=rte_getmodespk();
   int i,j;
@@ -114,11 +159,15 @@ on_save_keymap_clicked (GtkButton *button, gpointer user_data) {
   GList *list=NULL;
   gboolean update=FALSE;
 
-  const gchar *keymap_file=g_strdup_printf("%s/%sdefault.keymap",capable->home_dir,LIVES_CONFIG_DIR);
+  gchar *keymap_file=g_build_filename(capable->home_dir,LIVES_CONFIG_DIR,"default.keymap",NULL);
+  gchar *keymap_file2=g_build_filename(capable->home_dir,LIVES_CONFIG_DIR,"default.keymap2",NULL);
 
   if (button!=NULL) {
-    if (!do_warning_dialog_with_check_transient((_("\n\nClick 'OK' to save this keymap as your default\n\n")),0,GTK_WINDOW(rte_window))) return FALSE;
-
+    if (!do_warning_dialog_with_check_transient((_("\n\nClick 'OK' to save this keymap as your default\n\n")),0,GTK_WINDOW(rte_window))) {
+      g_free(keymap_file2);
+      g_free(keymap_file);
+      return FALSE;
+    }
     d_print ((tmp=g_strdup_printf(_("Saving keymap to %s\n"),keymap_file)));
     g_free(tmp);
   }
@@ -137,6 +186,8 @@ on_save_keymap_clicked (GtkButton *button, gpointer user_data) {
       g_free (msg);
       d_print_failed();
     }
+    g_free(keymap_file2);
+    g_free(keymap_file);
     return FALSE;
   }
 
@@ -146,7 +197,7 @@ on_save_keymap_clicked (GtkButton *button, gpointer user_data) {
     for (i=1;i<=prefs->rte_keys_virtual;i++) {
       for (j=0;j<modes;j++) {
 	if (rte_keymode_valid(i,j,TRUE)) {
-	  fputs(g_strdup_printf("%d|Weed%s\n",i,make_weed_hashname(rte_keymode_get_filter_idx(i,j))),kfile);
+	  fputs(g_strdup_printf("%d|Weed%s\n",i,make_weed_hashname(rte_keymode_get_filter_idx(i,j),TRUE)),kfile);
 	}
       }
     }
@@ -158,14 +209,25 @@ on_save_keymap_clicked (GtkButton *button, gpointer user_data) {
   }
 
   fclose (kfile);
+
+  // if we have default values, save as newer style
+  if (has_key_defaults()) {
+    save_keymap2_file(keymap_file2);
+  }
+  else unlink(keymap_file2);
+
+  g_free(keymap_file2);
+  g_free(keymap_file);
+
   d_print_done();
 
   return FALSE;
 }
 
 
-void
-on_save_rte_defs_activate (GtkMenuItem *menuitem, gpointer user_data) {
+
+
+void on_save_rte_defs_activate (GtkMenuItem *menuitem, gpointer user_data) {
   int fd,i;
   gint numfx;
   gchar *msg;
@@ -182,7 +244,7 @@ on_save_rte_defs_activate (GtkMenuItem *menuitem, gpointer user_data) {
   d_print(msg);
   g_free(msg);
 
-  if (!(fd=open(prefs->fxdefsfile,O_WRONLY|O_CREAT|O_TRUNC,S_IRUSR|S_IWUSR))) {
+  if ((fd=open(prefs->fxdefsfile,O_WRONLY|O_CREAT|O_TRUNC,S_IRUSR|S_IWUSR))==-1) {
     msg=g_strdup_printf (_("\n\nUnable to write defaults file\n%s\nError code %d\n"),prefs->fxdefsfile,errno);
     do_error_dialog (msg);
     g_free (msg);
@@ -201,7 +263,7 @@ on_save_rte_defs_activate (GtkMenuItem *menuitem, gpointer user_data) {
 
   close(fd);
 
-  if (!(fd=open(prefs->fxsizesfile,O_WRONLY|O_CREAT|O_TRUNC,S_IRUSR|S_IWUSR))) {
+  if ((fd=open(prefs->fxsizesfile,O_WRONLY|O_CREAT|O_TRUNC,S_IRUSR|S_IWUSR))==-1) {
     msg=g_strdup_printf (_("\n\nUnable to write default sizes file\n%s\nError code %d\n"),prefs->fxsizesfile,errno);
     do_error_dialog (msg);
     g_free (msg);
@@ -236,7 +298,7 @@ void load_rte_defs (void) {
   }
 
   if (g_file_test(prefs->fxdefsfile,G_FILE_TEST_EXISTS)) {
-    if (!(fd=open(prefs->fxdefsfile,O_RDONLY))) {
+    if ((fd=open(prefs->fxdefsfile,O_RDONLY))==-1) {
       msg2=g_strdup_printf (_("Unable to read defaults file\n%s\nError code %d\n"),prefs->fxdefsfile,errno);
       do_error_dialog (msg2);
       g_free (msg2);
@@ -268,7 +330,7 @@ void load_rte_defs (void) {
   }
 
   if (g_file_test(prefs->fxsizesfile,G_FILE_TEST_EXISTS)) {
-    if (!(fd=open(prefs->fxsizesfile,O_RDONLY))) {
+    if ((fd=open(prefs->fxsizesfile,O_RDONLY))==-1) {
       msg=g_strdup_printf (_("\n\nUnable to read sizes file\n%s\nError code %d\n"),prefs->fxsizesfile,errno);
       do_error_dialog (msg);
       g_free (msg);
@@ -300,131 +362,240 @@ void load_rte_defs (void) {
 }
 
 
+static void check_clear_all_button (void) {
+  int modes=rte_getmodespk();
+  int i,j;
+  gboolean hasone=FALSE;
+
+  for (i=0;i<prefs->rte_keys_virtual;i++) {
+    for (j=modes-1;j>=0;j--) {
+      if (rte_keymode_valid(i+1,j,TRUE)) hasone=TRUE;
+    }
+  }
+
+  gtk_widget_set_sensitive (GTK_WIDGET(clear_all_button), hasone);
+}
+
 
 
 gboolean on_load_keymap_clicked (GtkButton *button, gpointer user_data) {
-  // quick and dirty implementation
   int modes=rte_getmodespk();
   int i;
-  FILE *kfile;
+  int kfd;
+  int version,nparams;
+  int hlen;
+  FILE *kfile=NULL;
   gchar *msg,*tmp;
   gint key,mode;
   gchar buff[65536];
-  size_t linelen;
+  size_t linelen,bytes;
   gchar *whole=g_strdup (""),*whole2;
   GList *list=NULL,*new_list=NULL;
   gchar *hashname,*hashname_new=NULL;
   gboolean notfound=FALSE;
+  gboolean has_error=FALSE;
+  gboolean eof=FALSE;
   gint update=0;
   gchar *line=NULL;
   gchar *whashname;
 
-  gchar *keymap_file=g_strdup_printf("%s/%sdefault.keymap",capable->home_dir,LIVES_CONFIG_DIR);
-  gint count;
+  gchar *keymap_file=g_build_filename(capable->home_dir,LIVES_CONFIG_DIR,"default.keymap",NULL);
+  gchar *keymap_file2=g_build_filename(capable->home_dir,LIVES_CONFIG_DIR,"default.keymap2",NULL);
+
+  int *def_modes;
+
+  def_modes=(int *)g_malloc(prefs->rte_keys_virtual*sizint);
+  for (i=0;i<prefs->rte_keys_virtual;i++) def_modes[i]=-1;
+
+  if ((kfd=open (keymap_file2,O_RDONLY))!=-1) {
+    g_free(keymap_file);
+    keymap_file=keymap_file2;
+  }
+  else {
+    g_free(keymap_file2);
+    keymap_file2=NULL;
+  }
 
   msg=g_strdup_printf(_("Loading default keymap from %s..."),keymap_file);
   d_print(msg);
   g_free(msg);
 
-  if (!(kfile=fopen (keymap_file,"r"))) {
+  if (keymap_file2!=NULL) {
+    if ((kfd=open(keymap_file,O_RDONLY))==-1) has_error=TRUE;
+  }
+  else {
+    if (!(kfile=fopen (keymap_file,"r"))) {
+      has_error=TRUE;
+    }
+  }
+
+  if (has_error) {
     msg=g_strdup_printf (_("\n\nUnable to read from keymap file\n%s\nError code %d\n"),keymap_file,errno);
     g_free(keymap_file);
     do_error_dialog_with_check_transient (msg,FALSE,0,GTK_WINDOW(rte_window));
     g_free (msg);
     d_print_failed();
+    g_free(def_modes);
     return FALSE;
   }
 
-  g_free(keymap_file);
+  on_clear_all_clicked(NULL,user_data);
+  if (mainw->error) {
+    mainw->error=FALSE;
+    d_print_cancelled();
+    g_free(def_modes);
+    return FALSE;
+  }
 
-  while (fgets(buff,65536,kfile)) {
-    if (buff!=NULL) {
-      // TODO - should we g_free() line ?
-      line=(g_strchomp (g_strchug(buff)));
-      if ((linelen=strlen (line))) {
-	whole2=g_strconcat (whole,line,NULL);
-	if (whole2!=whole) g_free (whole);
-	whole=whole2;
-	if (linelen<(size_t)65535) {
-	  list=g_list_append (list, g_strdup (whole));
-	  g_free (whole);
-	  whole=g_strdup ("");
+  if (keymap_file2==NULL) {
+    while (fgets(buff,65536,kfile)) {
+      if (buff!=NULL) {
+	line=(g_strchomp (g_strchug(buff)));
+	if ((linelen=strlen (line))) {
+	  whole2=g_strconcat (whole,line,NULL);
+	  if (whole2!=whole) g_free (whole);
+	  whole=whole2;
+	  if (linelen<(size_t)65535) {
+	    list=g_list_append (list, g_strdup (whole));
+	    g_free (whole);
+	    whole=g_strdup ("");
+	  }
 	}
       }
     }
+    fclose (kfile);
+
+    if (!strcmp(g_list_nth_data(list,0),"LiVES keymap file version 2")||!strcmp(g_list_nth_data(list,0),"LiVES keymap file version 1")) update=1;
+    if (!strcmp(g_list_nth_data(list,0),"LiVES keymap file version 3")) update=2;
   }
+  else {
+    // read version
+    bytes=read(kfd,&version,sizint);
+    if (bytes<sizint) {
+      eof=TRUE;
+    }
+  }
+
   g_free (whole);
-  fclose (kfile);
 
-  on_clear_all_clicked(NULL,user_data);
-
-  if (!strcmp(g_list_nth_data(list,0),"LiVES keymap file version 2")||!strcmp(g_list_nth_data(list,0),"LiVES keymap file version 1")) update=1;
-  if (!strcmp(g_list_nth_data(list,0),"LiVES keymap file version 3")) update=2;
-  
-  for (i=1;i<g_list_length(list);i++) {
+  for (i=1;(keymap_file2==NULL&&i<g_list_length(list))||(keymap_file2!=NULL&&!eof);i++) {
     gchar **array;
-    line=(gchar *)g_list_nth_data(list,i);
+
+    if (keymap_file2==NULL) {
+      line=(gchar *)g_list_nth_data(list,i);
     
-    if (get_token_count(line,'|')<2) {
-      d_print((tmp=g_strdup_printf(_("Invalid line %d in %s\n"),i,keymap_file)));
-      g_free(tmp);
-      continue;
-    }
-
-    count=get_token_count(line,'|');
-    array=g_strsplit (line,"|",-1);
-
-    if (count>1&&!strcmp(array[0],"defaults")) {
-      g_strfreev(array);
-      array=g_strsplit(line,"|",2);
-      if (prefs->fxdefsfile!=NULL) g_free(prefs->fxdefsfile);
-      prefs->fxdefsfile=g_strdup(array[1]);
-      g_strfreev(array);
-      continue;
-    }
-
-    if (count>1&&!strcmp(array[0],"sizes")) {
-      g_strfreev(array);
-      array=g_strsplit(line,"|",2);
-      if (prefs->fxsizesfile!=NULL) g_free(prefs->fxsizesfile);
-      prefs->fxsizesfile=g_strdup(array[1]);
-      g_strfreev(array);
-      continue;
-    }
-
-    key=atoi(array[0]);
+      if (get_token_count(line,'|')<2) {
+	d_print((tmp=g_strdup_printf(_("Invalid line %d in %s\n"),i,keymap_file)));
+	g_free(tmp);
+	continue;
+      }
+      
+      array=g_strsplit (line,"|",-1);
+      
+      if (!strcmp(array[0],"defaults")) {
+	g_strfreev(array);
+	array=g_strsplit(line,"|",2);
+	if (prefs->fxdefsfile!=NULL) g_free(prefs->fxdefsfile);
+	prefs->fxdefsfile=g_strdup(array[1]);
+	g_strfreev(array);
+	continue;
+      }
+      
+      if (!strcmp(array[0],"defaults")) {
+	g_strfreev(array);
+	array=g_strsplit(line,"|",2);
+	if (prefs->fxdefsfile!=NULL) g_free(prefs->fxdefsfile);
+	prefs->fxdefsfile=g_strdup(array[1]);
+	g_strfreev(array);
+	continue;
+      }
+      
+      if (!strcmp(array[0],"sizes")) {
+	g_strfreev(array);
+	array=g_strsplit(line,"|",2);
+	if (prefs->fxsizesfile!=NULL) g_free(prefs->fxsizesfile);
+	prefs->fxsizesfile=g_strdup(array[1]);
+	g_strfreev(array);
+	continue;
+      }
+      
+      key=atoi(array[0]);
     
-    hashname=line+1+strlen(array[0]);
-    g_strfreev(array);
+      hashname=g_strdup(array[1]);
+      g_strfreev(array);
+      
+      if (update>0) {
+	if (update==1) hashname_new=g_strdup_printf("%d|Weed%s1\n",key,hashname);
+	if (update==2) hashname_new=g_strdup_printf("%d|Weed%s\n",key,hashname);
+	new_list=g_list_append(new_list,hashname_new);
+	g_free(hashname);
+	continue;
+      }
+    }
+    else {
+      //read key and hashname
+      bytes=read(kfd,&key,sizint);
+      if (bytes<sizint) {
+	eof=TRUE;
+	break;
+      }
+
+      bytes=read(kfd,&hlen,sizint);
+      if (bytes<sizint) {
+	eof=TRUE;
+	break;
+      }
+
+      hashname=g_malloc(hlen+1);
+
+      bytes=read(kfd,hashname,hlen);
+      if (bytes<hlen) {
+	eof=TRUE;
+	g_free(hashname);
+	break;
+      }
+
+      memset(hashname+hlen,0,1);
+
+      array=g_strsplit(hashname,"|",-1);
+      g_free(hashname);
+      hashname=g_strdup(array[0]);
+      g_strfreev(array);
 
-    if (update>0) {
-      if (update==1) hashname_new=g_strdup_printf("%d|Weed%s1\n",key,hashname);
-      if (update==2) hashname_new=g_strdup_printf("%d|Weed%s\n",key,hashname);
-      new_list=g_list_append(new_list,hashname_new);
-      continue;
     }
 
     if (key<1||key>prefs->rte_keys_virtual) {
       d_print((tmp=g_strdup_printf(_("Invalid key %d in %s\n"),key,keymap_file)));
       g_free(tmp);
       notfound=TRUE;
+      g_free(hashname);
       continue;
     }
 
+    def_modes[key-1]++;
+
+
     if (strncmp(hashname,"Weed",4)||strlen(hashname)<5) {
       d_print((tmp=g_strdup_printf(_("Invalid effect %s in %s\n"),hashname,keymap_file)));
       g_free(tmp);
       notfound=TRUE;
+      g_free(hashname);
       continue;
     }
 
+    // ignore "Weed"
     whashname=hashname+4;
 
     if ((mode=weed_add_effectkey(key,whashname,TRUE))==-1) {
       // could not locate effect
+      d_print((tmp=g_strdup_printf(_("Unknown effect %s in %s\n"),whashname,keymap_file)));
       notfound=TRUE;
+      g_free(hashname);
       continue;
     }
+
+    g_free(hashname);
+
     if (mode==-2){
       d_print((tmp=g_strdup_printf(_("This version of LiVES cannot mix generators/non-generators on the same key (%d) !\n"),key)));
       g_free(tmp);
@@ -435,28 +606,52 @@ gboolean on_load_keymap_clicked (GtkButton *button, gpointer user_data) {
       g_free(tmp);
       continue;
     }
-    else if (rte_window!=NULL) {
+    if (rte_window!=NULL) {
       gtk_entry_set_text (GTK_ENTRY(combo_entries[(key-1)*modes+mode]),(tmp=rte_keymode_get_filter_name(key,mode)));
       g_free(tmp);
       type_label_set_text(key-1,mode);
     }
+
+
+    if (keymap_file2!=NULL) {
+      // read param defaults
+      bytes=read(kfd,&nparams,sizint);
+      if (bytes<sizint) {
+	eof=TRUE;
+	break;
+      }
+      if (nparams>0) {
+	read_key_defaults(kfd,nparams,key-1,def_modes[key-1],version);
+      }
+    }
   }
 
-  g_list_free_strings(list);
-  g_list_free(list);
+  g_free(keymap_file);
 
-  if (update>0) {
-    d_print(_("update required.\n"));
-    on_save_keymap_clicked(NULL,new_list);
-    g_list_free_strings(new_list);
-    g_list_free(new_list);
-    on_load_keymap_clicked(NULL,NULL);
+  if (keymap_file2==NULL) {
+    g_list_free_strings(list);
+    g_list_free(list);
+
+    if (update>0) {
+      d_print(_("update required.\n"));
+      on_save_keymap_clicked(NULL,new_list);
+      g_list_free_strings(new_list);
+      g_list_free(new_list);
+      on_load_keymap_clicked(NULL,NULL);
+    }
+    else d_print_done();
+  }
+  else {
+    close(kfd);
+    d_print_done();
   }
-  else d_print_done();
-    
-  if (notfound&&mainw->is_ready) do_warning_dialog_with_check_transient(_("\n\nSome effects could not be located.\n\n"),0,GTK_WINDOW(rte_window));
 
-  if (!mainw->is_ready) load_rte_defs();
+  if (mainw->is_ready) {
+    check_clear_all_button();
+    if (notfound) do_warning_dialog_with_check_transient(_("\n\nSome effects could not be located.\n\n"),0,GTK_WINDOW(rte_window));
+  }
+  else load_rte_defs();
+  g_free(def_modes);
 
   return FALSE;
 }
@@ -511,7 +706,7 @@ on_rte_info_clicked (GtkButton *button, gpointer user_data) {
 
   gtk_container_set_border_width (GTK_CONTAINER (rte_info_window), 40);
   gtk_window_set_transient_for(GTK_WINDOW(rte_info_window),GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(button))));
-  gtk_window_set_position (GTK_WINDOW (rte_info_window), GTK_WIN_POS_CENTER);
+  gtk_window_set_position (GTK_WINDOW (rte_info_window), GTK_WIN_POS_CENTER_ALWAYS);
   gtk_window_set_default_size (GTK_WINDOW (rte_info_window), RTE_INFO_WIDTH, RTE_INFO_HEIGHT);
 
   gtk_widget_show(rte_info_window);
@@ -616,25 +811,9 @@ on_rte_info_clicked (GtkButton *button, gpointer user_data) {
 
 
 
-void check_clear_all_button (void) {
-  int modes=rte_getmodespk();
-  int i,j;
-  gboolean hasone=FALSE;
-
-  for (i=0;i<prefs->rte_keys_virtual;i++) {
-    for (j=modes-1;j>=0;j--) {
-      if (rte_keymode_valid(i+1,j,TRUE)) hasone=TRUE;
-    }
-  }
-
-  gtk_widget_set_sensitive (GTK_WIDGET(clear_all_button), hasone);
-}
 
 
-
-
-void 
-on_clear_clicked (GtkButton *button, gpointer user_data) {
+void on_clear_clicked (GtkButton *button, gpointer user_data) {
   // this is for the "delete" buttons, c.f. clear_all
 
   gint idx=GPOINTER_TO_INT(user_data);
@@ -676,7 +855,13 @@ void on_params_clicked (GtkButton *button, gpointer user_data) {
   weed_plant_t *inst;
   lives_rfx_t *rfx;
 
-  if ((inst=rte_keymode_get_instance(key+1,mode))==NULL) return;
+  if ((inst=rte_keymode_get_instance(key+1,mode))==NULL) {
+    weed_plant_t *filter=rte_keymode_get_filter(key+1,mode);
+    if (filter==NULL) return;
+    inst=weed_instance_from_filter(filter);
+    apply_key_defaults(inst,key,mode);
+  }
+  else weed_instance_ref(inst);
 
   if (fx_dialog[1]!=NULL) {
     rfx=g_object_get_data (G_OBJECT (fx_dialog[1]),"rfx");
@@ -689,6 +874,10 @@ void on_params_clicked (GtkButton *button, gpointer user_data) {
   keyw=key;
   modew=mode;
   on_render_fx_pre_activate(NULL,rfx);
+
+  // record the key so we know whose parameters to record later
+  weed_set_int_value(rfx->source,"host_hotkey",key);
+
   g_object_set_data (G_OBJECT (fx_dialog[1]),"key",GINT_TO_POINTER (key));
   g_object_set_data (G_OBJECT (fx_dialog[1]),"mode",GINT_TO_POINTER (mode));
   g_object_set_data (G_OBJECT (fx_dialog[1]),"rfx",rfx);
@@ -748,8 +937,7 @@ void do_mix_error(void) {
 
 
 
-void 
-fx_changed (GtkItem *item, gpointer user_data) {
+void fx_changed (GtkItem *item, gpointer user_data) {
   gint key_mode=GPOINTER_TO_INT(user_data);
   int modes=rte_getmodespk();
   gint key=(gint)(key_mode/modes);
@@ -1035,10 +1223,11 @@ GtkWidget * create_rte_window (void) {
       // fill names of our effects
       fx_idx=0;
       list=name_type_list;
-
+      
       while (list!=NULL) {
-	weed_plant_t *filter=get_weed_filter(weed_get_idx_for_hashname(g_list_nth_data(hash_list,fx_idx),FALSE));
-	if ((enabled_in_channels(filter,FALSE)>1&&!has_video_chans_in(filter,FALSE))||(weed_plant_has_leaf(filter,"host_menu_hide")&&weed_get_boolean_value(filter,"host_menu_hide",&error)==WEED_TRUE)) {
+	weed_plant_t *filter=get_weed_filter(weed_get_idx_for_hashname(g_list_nth_data(hash_list,fx_idx),TRUE));
+	int filter_flags=weed_get_int_value(filter,"flags",&error);
+	if ((weed_plant_has_leaf(filter,"plugin_unstable")&&weed_get_boolean_value(filter,"plugin_unstable",&error)==WEED_TRUE&&!prefs->unstable_fx)||((enabled_in_channels(filter,FALSE)>1&&!has_video_chans_in(filter,FALSE))||(weed_plant_has_leaf(filter,"host_menu_hide")&&weed_get_boolean_value(filter,"host_menu_hide",&error)==WEED_TRUE)||(filter_flags&WEED_FILTER_IS_CONVERTER))) {
 	  list = list->next;
 	  fx_idx++;
 	  continue; // skip audio transitions and hidden entries
@@ -1057,11 +1246,11 @@ GtkWidget * create_rte_window (void) {
 	list = list->next;
 	fx_idx++;
       }
-
+      
       gtk_entry_set_text (GTK_ENTRY (combo_entries[idx]),(tmp=rte_keymode_get_filter_name(i+1,j)));
       g_free(tmp);
       gtk_entry_set_editable (GTK_ENTRY (combo_entries[idx]), FALSE);
-
+      
       hbox = gtk_hbox_new (FALSE, 12);
       gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
@@ -1149,6 +1338,7 @@ GtkWidget * create_rte_window (void) {
   g_signal_connect (GTK_OBJECT (load_keymap_button), "clicked",
 		    G_CALLBACK (on_load_keymap_clicked),
 		    GINT_TO_POINTER(1));
+
   g_signal_connect (GTK_OBJECT (clear_all_button), "clicked",
 		    G_CALLBACK (on_clear_all_clicked),
 		    GINT_TO_POINTER(1));
@@ -1214,14 +1404,8 @@ void rtew_set_mode_radio (gint key, gint mode) {
 }
 
 
-void rtew_set_param_button (gint key, gint mode, gboolean on) {
-  int modes=rte_getmodespk();
-  gtk_widget_set_sensitive(param_buttons[key*modes+mode],on);
 
-}
-
-
-void close_pwindow (gint key, gint mode, gboolean is_reinit) {
+void redraw_pwindow (gint key, gint mode) {
   gint keyw=0,modew=0;
   GList *child_list;
   int i;
@@ -1234,26 +1418,20 @@ void close_pwindow (gint key, gint mode, gboolean is_reinit) {
       modew=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (fx_dialog[1]),"mode"));
     }
     if (rfx->is_template||(key==keyw&&mode==modew)) {
-      if (!is_reinit&&!rfx->is_template) {
-	// destroy window
-	gtk_widget_destroy(fx_dialog[1]);
-	on_paramwindow_cancel_clicked2(NULL,rfx);
-      }
-      else {
-	// or just rip out the contents
-	if (mainw->invis==NULL) mainw->invis=gtk_vbox_new(FALSE,0);
-	child_list=gtk_container_get_children(GTK_CONTAINER(GTK_DIALOG(fx_dialog[1])->vbox));
-	for (i=0;i<g_list_length(child_list);i++) {
-	  GtkWidget *widget=g_list_nth_data(child_list,i);
-	  if (widget!=GTK_DIALOG (fx_dialog[1])->action_area) {
-	    // we have to do this, because using gtk_widget_destroy() here can causes a crash [bug in gtk+ ???]
-	    gtk_widget_reparent (widget,mainw->invis);
-	  }
+      // rip out the contents
+      if (mainw->invis==NULL) mainw->invis=gtk_vbox_new(FALSE,0);
+      child_list=gtk_container_get_children(GTK_CONTAINER(GTK_DIALOG(fx_dialog[1])->vbox));
+      for (i=0;i<g_list_length(child_list);i++) {
+	GtkWidget *widget=g_list_nth_data(child_list,i);
+	if (widget!=GTK_DIALOG (fx_dialog[1])->action_area) {
+	  // we have to do this, because using gtk_widget_destroy() here 
+	  // can causes a crash [bug in gtk+ ???]
+	  gtk_widget_reparent (widget,mainw->invis);
 	}
-	if (child_list!=NULL) g_list_free(child_list);
-	on_paramwindow_cancel_clicked(NULL,NULL);
-	restore_pwindow(rfx);
       }
+      if (child_list!=NULL) g_list_free(child_list);
+      on_paramwindow_cancel_clicked(NULL,NULL);
+      restore_pwindow(rfx);
     }
   }
 }
@@ -1304,6 +1482,20 @@ void rte_set_defs_activate (GtkMenuItem *menuitem, gpointer user_data) {
 
 
 
+void rte_set_key_defs (GtkButton *button, lives_rfx_t *rfx) {
+  gint key,mode;
+  if (mainw->textwidget_focus!=NULL) after_param_text_changed(mainw->textwidget_focus,rfx);
+
+  if (rfx->num_params>0) {
+    key=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (fx_dialog[1]),"key"));
+    mode=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (fx_dialog[1]),"mode"));
+    set_key_defaults(rfx->source,key,mode);
+  }
+}
+
+
+
+
 void rte_set_defs_ok (GtkButton *button, lives_rfx_t *rfx) {
   int i;
   weed_plant_t **ptmpls,*filter,*copy_param=NULL;
@@ -1334,6 +1526,9 @@ void rte_set_defs_ok (GtkButton *button, lives_rfx_t *rfx) {
 	break;
       case LIVES_PARAM_BOOL:
 	weed_set_boolean_array(ptmpls[i],"host_default",1,rfx->params[i].value);
+	break;
+      default:
+	break;
       }
     }
     weed_free(ptmpls);
@@ -1357,7 +1552,7 @@ void load_default_keymap(void) {
   gchar *keymap_template=g_strdup_printf("%s%sdefault.keymap",prefs->prefix_dir,DATA_DIR);
   gchar *com,*tmp;
 
-  pthread_mutex_lock(&mainw->gtk_mutex);
+  threaded_dialog_spin();
   if (!g_file_test (keymap_file, G_FILE_TEST_EXISTS)) {
     com=g_strdup_printf("/bin/cp %s %s",keymap_template,keymap_file);
     dummyvar=system(com);
@@ -1369,11 +1564,11 @@ void load_default_keymap(void) {
     g_free(tmp);
     g_free(keymap_file);
     g_free(keymap_template);
-    pthread_mutex_unlock(&mainw->gtk_mutex);
+    threaded_dialog_spin();
     return;
   }
   on_load_keymap_clicked(NULL,NULL);
   g_free(keymap_file);
   g_free(keymap_template);
-  pthread_mutex_unlock(&mainw->gtk_mutex);
+  threaded_dialog_spin();
 }
